let min_array cmp tab =
  let l = Array.length tab in
  if l = 0 then raise Not_found
  else
    let min = ref tab.(0) in
    for i = 1 to l - 1 do
      if cmp tab.(i) !min < 0 then min := tab.(i)
    done;
    !min

(* Fonction utilitaire pour les tests *)
let array_to_string pr tab =
  let l = Array.to_list tab in
  "[|" ^ String.concat "; " (List.map pr l) ^ "|]"

let int_array n = Array.init n (fun i -> i)

(* Test min_array *)
let () =
  let t = int_array 10 in
  t.(5) <- -2;
  let m = min_array Int.compare t in
  Printf.printf "Valeur min de %s = %d\n" (array_to_string string_of_int t) m

let is_sorted cmp tab =
  let i = ref 1 in
  let len = Array.length tab in
  while !i < len && cmp tab.(!i - 1) tab.(!i) <= 0 do
    i := !i + 1
  done;
  !i >= len

(* test is_sorted *)
let () =
  let t = int_array 10 in
  let b = is_sorted Int.compare t in
  Printf.printf "is_sorted de %s = %b\n" (array_to_string string_of_int t) b;
  t.(8) <- -1;
  let b = is_sorted Int.compare t in
  Printf.printf "is_sorted de %s = %b\n" (array_to_string string_of_int t) b

let reverse tab =
  let len = Array.length tab in
  for i = 0 to len / 2 - 1 do
    let tmp = tab.(i) in
    tab.(i) <- tab.(len - i - 1);
    tab.(len - i - 1) <- tmp
  done

(* Fonction de test pour les fonctions modifiant un tableau *)

let test_fun msg f t =
  Printf.printf "%s %s = " msg (array_to_string string_of_int t);
  f t;
  Printf.printf "%s\n" (array_to_string string_of_int t)

let () =
  test_fun "reverse" reverse [||];
  test_fun "reverse" reverse (int_array 10);
  test_fun "reverse" reverse (int_array 11)

let average tab =
  let l = Array.length tab in
  if l = 0 then None else Some (Array.fold_left ( +. ) 0.0 tab /. float l)

let () =
  let t = Array.map float (int_array 10) in
  let a = average t in
  Printf.printf "average  %s = %s\n"
    (array_to_string string_of_float t)
    (match a with
    | None -> "None"
    | Some f -> "Some " ^ string_of_float f)

let rand_interval a b = a + Random.int (b - a)

let shuffle tab =
  let l = Array.length tab in
  for i = 0 to l - 2 do
    let j = rand_interval i l in
    let tmp = tab.(i) in
    tab.(i) <- tab.(j);
    tab.(j) <- tmp
  done

let () = test_fun "shuffle" shuffle (int_array 10)

let rotate tab =
  let l = Array.length tab in
  if l > 0 then begin
    let tmp = tab.(l-1) in
    for i = l - 1 downto 1 do
      tab.(i) <- tab.(i-1)
    done;
    tab.(0) <- tmp
  end

let () =
  let t = int_array 10 in
  test_fun "rotate" rotate t;
  test_fun "rotate" rotate t;
  test_fun "rotate" rotate t;
  test_fun "rotate" rotate t

