type 'a cell = {
  mutable value : 'a;
  mutable next : 'a cell option;
}

type 'a t = {
  mutable first : 'a cell option;
  mutable last : 'a cell option;
  mutable size : int;
}

let empty () = { first = None; last = None; size = 0 }

let push s v =
  let cell = Some { value = v; next = None } in
  s.size <- s.size + 1;
  match s.last with
  | Some c ->
      c.next <- cell;
      s.last <- cell
  | None ->
      s.first <- cell;
      s.last <- cell

let pop s =
  match s.first with
  | None -> failwith "Lifo.pop"
  | Some cell ->
      s.first <- cell.next;
      if s.first = None then s.last <- None;
      s.size <- s.size - 1;
      cell.value

let peek s =
  match s.first with
  | None -> failwith "Lifo.peek"
  | Some { value; _ } -> value

let size s = s.size

let iter f s =
  let rec loop l =
    match l with
    | None -> ()
    | Some cell ->
        f cell.value;
        loop cell.next
  in
  loop s.first
