type 'a t =
  | Leaf
  | Node of 'a t * 'a * 'a t


let is_node t =
  match t with
  Leaf -> false
  | Node _ -> true

let has_left t =
  match t with
  Node (Node _, _, _) -> true
  | _ -> false

let get_value t =
  match t with
  Leaf -> failwith "get_value"
  | Node (_, v, _) -> v

let get_left t =
  match t with
  Node (l, _, _) -> l
  | _ -> failwith "get_left"

let left_most t =
  let c = ref t in
  while is_node !c && has_left !c do
    c := get_left !c;
  done;
  get_value !c



module Iter (Col : Defn.S) = struct
  let iter f t =
    let todo = Col.empty () in
    Col.push todo t;
    while Col.size todo > 0 do
      match Col.pop todo with
      | Leaf -> ()
      | Node (left, v, right) ->
          f v;
          Col.push todo right;
          Col.push todo left
    done
end
